﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace HeuristicLab.Common {
  public static class Extensions {
    public static bool IsAlmost(this double x, double y) {
      return Math.Abs(x - y) < 1.0e-6;
    }

    public static T SelectRandom<T>(this IEnumerable<T> xs, Random rand) {
      var xsArr = xs.ToArray();
      return xsArr[rand.Next(xsArr.Length)];
    }

    public static IEnumerable<T> SampleProportional<T>(this IEnumerable<T> source, Random random, IEnumerable<double> weights) {
      var sourceArray = source.ToArray();
      var valueArray = weights.ToArray();
      double total = valueArray.Sum();

      while (true) {
        int index = 0;
        double ball = valueArray[index], sum = random.NextDouble() * total;
        while (ball < sum)
          ball += valueArray[++index];
        yield return sourceArray[index];
      }
    }
  }
}
